# $Id: Config.pm,v 1.2 2004/06/20 15:49:29 rich Exp $

package Subscripto::Config;

use File::stat;
use Fcntl ':mode';
use Carp;
use IO::File;
use XML::LibXML;
use Subscripto::Address;

sub new
{
    my $class = shift(@_);
    my $dir = shift(@_);
    my $self = bless({}, $class);

    if (!defined($dir))
    {
	$dir = $ENV{HOME}.'/.subscripto';
    }

    # Check the permissions on $dir. If they're not 0700, barf.
    if (-d $dir)
    {
	my $sb = stat($dir);
	if ($sb->mode & (S_IRWXG|S_IRWXO))
	{
	    carp
		'Refusing to start - unsafe permissions on '
		.$dir.' - use 0700';
	}
    }

    $self->_clear();

    # Set up defaults.
    $self->{dir} = $dir;
    $self->{file} = $dir.'/config.xml';

    # If the file exists and this fails, abort. Otherwise, ignore it.
    if (!$self->load() && -r $self->{file})
    {
	carp 'Unable to parse configuration file "'.$file.'"';
    }

    return $self;
}

sub _clear
{
    my $self = shift(@_);

    $self->{dir} = undef;
    $self->{file} = undef;

    $self->{xml} = undef;
    $self->{xmlparser} = undef;

    $self->{folders} = [];

    $self->{addrs} = [];
    $self->{inactive_addrs} = {};
}

sub load
{
    my $self = shift(@_);
    my $fh = new IO::File('<'.$self->{file});
    my $ret = 0; # Fail by default.

    if (defined($fh))
    {
	$self->{xmlparser} = XML::LibXML->new();
	if (defined($self->{xmlparser}))
	{
	    $self->{xml} = $self->{xmlparser}->parse_fh($fh);
	}
	$fh->close();
    }

    return $self->_parse();
}

sub _parse
{
    my $self = shift(@_);
    my $ret = 1; # Succeed by default.

    if (defined($self->{xml}))
    {
	# Parse the config.

	# Mailboxes.
	my $match = $self->{xml}->findnodes('/subscripto/sources/mailbox');
	my $node;

	foreach $node ($match->get_nodelist())
	{
	    push(@{$self->{folders}}, $node->textContent());
	}

	# Addresses;
	$match = $self->{xml}->findnodes('/subscripto/addresses/address');

	foreach $node ($match->get_nodelist())
	{
	    my $addr = $node->textContent();
	    my @attrs = $node->attributes();
	    my $attr;
	    my $inactive = 0;

	    foreach $attr (@attrs)
	    {
		$inactive
		    = ($attr->nodeName eq 'status')
		    && ($attr->textContent() eq 'inactive');
	    }

	    push(@{$self->{addrs}}, new Subscripto::Address($addr, $inactive));
	    if ($inactive)
	    {
		$self->{inactive_addrs}->{$addr} = 1;
	    }
	}
    }
    else
    {
	$ret = 0;
    }

    return $ret;
}

sub reload
{
    my $self = shift(@_);

    $self->_clear();
    return $self->load();
}

sub save
{
    my $self = shift(@_);

    -d $self->{dir}
        || mkdir($self->{dir}, 0700)
	|| croak 'Unable to create '.$self->{dir};

    # TODO: Write the configuration.
}

sub get_folders
{
    my $self = shift(@_);

    return @{$self->{folders}};
}

sub is_inactive
{
    my $self = shift(@_);
    my $addr = shift(@_);

    return defined($self->{inactive_addrs}->{$addr});
}

1;
